@@ -1,4 +1,5 @@
 use std::cmp::Ordering;
+use std::convert::Infallible;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::iter::{Product, Sum};
@@ -6,6 +7,7 @@ use std::ops::*;
 use std::str::FromStr;
 
 use async_trait::async_trait;
+use bytes::Bytes;
 use collate::Collate;
 use destream::{Decoder, EncodeSeq, Encoder, FromStream, IntoStream, ToStream};
 use futures::TryFutureExt;
@@ -90,6 +92,16 @@ impl Default for Boolean {
     }
 }
 
+#[async_trait]
+impl async_hash::Hash for Boolean {
+    type Context = ();
+    type Error = Infallible;
+
+    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
+        async_hash::Hash::hash(&self.0, cxt).await
+    }
+}
+
 impl From<bool> for Boolean {
     fn from(b: bool) -> Boolean {
         Self(b)
@@ -374,6 +386,19 @@ impl FloatInstance for Complex {
     }
 }
 
+#[async_trait]
+impl async_hash::Hash for Complex {
+    type Context = ();
+    type Error = Infallible;
+
+    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
+        match self {
+            Self::C32(c) => async_hash::Hash::hash(&[c.re, c.im], cxt).await,
+            Self::C64(c) => async_hash::Hash::hash(&[c.re, c.im], cxt).await,
+        }
+    }
+}
+
 impl CastFrom<[f32; 2]> for Complex {
     fn cast_from(arr: [f32; 2]) -> Self {
         Self::C32(num::Complex::new(arr[0], arr[1]))
@@ -1083,6 +1108,19 @@ impl PartialOrd for Float {
     }
 }
 
+#[async_trait]
+impl async_hash::Hash for Float {
+    type Context = ();
+    type Error = Infallible;
+
+    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
+        match self {
+            Self::F32(f) => async_hash::Hash::hash(f, cxt).await,
+            Self::F64(f) => async_hash::Hash::hash(f, cxt).await,
+        }
+    }
+}
+
 impl Default for Float {
     fn default() -> Float {
         Float::F32(f32::default())
@@ -1643,6 +1681,7 @@ impl PartialOrd for Int {
 impl Ord for Int {
     fn cmp(&self, other: &Self) -> Ordering {
         match (self, other) {
+            (Int::I8(l), Int::I8(r)) => l.cmp(r),
             (Int::I16(l), Int::I16(r)) => l.cmp(r),
             (Int::I32(l), Int::I32(r)) => l.cmp(r),
             (Int::I64(l), Int::I64(r)) => l.cmp(r),
@@ -1651,6 +1690,21 @@ impl Ord for Int {
     }
 }
 
+#[async_trait]
+impl async_hash::Hash for Int {
+    type Context = ();
+    type Error = Infallible;
+
+    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
+        match self {
+            Self::I8(i) => async_hash::Hash::hash(i, cxt).await,
+            Self::I16(i) => async_hash::Hash::hash(i, cxt).await,
+            Self::I32(i) => async_hash::Hash::hash(i, cxt).await,
+            Self::I64(i) => async_hash::Hash::hash(i, cxt).await,
+        }
+    }
+}
+
 impl Default for Int {
     fn default() -> Int {
         Int::I16(i16::default())
@@ -2199,6 +2253,21 @@ impl Ord for UInt {
     }
 }
 
+#[async_trait]
+impl async_hash::Hash for UInt {
+    type Context = ();
+    type Error = Infallible;
+
+    async fn hash(&self, cxt: &Self::Context) -> Result<Bytes, Self::Error> {
+        match self {
+            Self::U8(u) => async_hash::Hash::hash(u, cxt).await,
+            Self::U16(u) => async_hash::Hash::hash(u, cxt).await,
+            Self::U32(u) => async_hash::Hash::hash(u, cxt).await,
+            Self::U64(u) => async_hash::Hash::hash(u, cxt).await,
+        }
+    }
+}
+
 impl Default for UInt {
     fn default() -> UInt {
         UInt::U8(u8::default())
